package com.jz.common.utils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.io.*;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.zip.*;

public class ZipUtils {
	
	
	private static final Logger logger = LoggerFactory.getLogger(ZipUtils.class);

	
	/**
	 * 列出zip压缩包中的文件
	 * @param toRemoveFileNameMap 返回结果里要删除的
	 * @param zipPathName
	 * @throws Exception
	 */
	public static Map<String, Integer> listZipFile(Map<String, Integer> toRemoveFileNameMap, String zipPathName,String suffix){
		Map<String, Integer> fileNameMap = new HashMap<>();
		ZipFile zipFile = null;
		try {
			File file = new File(zipPathName);
			if (!file.exists()) {
				return fileNameMap;
			}
			zipFile = new ZipFile(file);
			Enumeration<? extends ZipEntry> zipEntrys = zipFile.entries();
			while(zipEntrys.hasMoreElements()){
				ZipEntry zipEntry = zipEntrys.nextElement();
				String fileName = zipEntry.getName();
				if (!StringUtils.isEmpty(fileName) && fileName.endsWith(suffix)) {
					fileName = fileName.substring(0, fileName.lastIndexOf(suffix));
					if (CollectionUtils.isEmpty(toRemoveFileNameMap)) {
						fileNameMap.put(fileName, 1);
					}else {
						if(!toRemoveFileNameMap.containsKey(fileName)) {
							fileNameMap.put(fileName, 1);
						}
					}
				}
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		}finally {
			if (zipFile != null) {
				try {
					zipFile.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return fileNameMap;		
	}

	/**
	 * 递归压缩文件夹
	 * 
	 * @param srcRootDir
	 *            压缩文件夹根目录的子路径
	 * @param file
	 *            当前递归压缩的文件或目录对象
	 * @param zos
	 *            压缩文件存储对象
	 * @throws Exception
	 */
	private static void zip(String srcRootDir, File file, ZipOutputStream zos) throws IOException {
		if (file == null) {
			return;
		}

		// 如果是文件，则直接压缩该文件
		if (file.isFile()) {
			int count, bufferLen = 1024;
			byte data[] = new byte[bufferLen];

			// 获取文件相对于压缩文件夹根目录的子路径
			String subPath = file.getAbsolutePath();
			int index = subPath.indexOf(srcRootDir);
			if (index != -1) {
				subPath = subPath.substring(srcRootDir.length() + File.separator.length());
			}
				
			BufferedInputStream bis = null ;
			FileInputStream fis = null;
			try {
				ZipEntry entry = new ZipEntry(file.getName());
				zos.putNextEntry(entry);
				fis = new FileInputStream(file);
				bis = new BufferedInputStream(fis);
				while ((count = bis.read(data, 0, bufferLen)) != -1) {
					zos.write(data, 0, count);
				}
			}finally {
				
				if (bis != null) {
					try {
						bis.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				if (fis != null) {
					try {
						fis.close();
					} catch (IOException e) {
						e.printStackTrace();
					}
				}
				try {
					zos.closeEntry();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			
			
		}
		// 如果是目录，则压缩整个目录
		else {
			// 压缩目录中的文件或子目录
			File[] childFileList = file.listFiles();
			for (int n = 0; n < childFileList.length; n++) {
				
				childFileList[n].getAbsolutePath().indexOf(file.getAbsolutePath());
				zip(srcRootDir, childFileList[n], zos);
			}
		}
	}

	/**
	 * 对文件或文件目录进行压缩
	 * 
	 * @param srcPath  要压缩的目录 
	 * @param zipTargetDirPath  压缩包文件所保存的路径。注意：zipPath不能是srcPath路径下的子文件夹
	 * @param zipFileName  压缩包文件名
	 * @throws Exception
	 */
	public static void zip(String projectSourcePath, String zipTargetDirPath, String zipTargetFileName) {
		
		// 创建压缩文件保存的文件对象
		String zipTargetFilePath = zipTargetDirPath + zipTargetFileName;
		
		CheckedOutputStream cos = null;
		ZipOutputStream zos = null;
		File zipTargetFile = new File(zipTargetFilePath);
		File backupFile = new File(zipTargetFilePath+".backup");
		FileOutputStream fos = null ;
		try {
			
			// 判断压缩文件保存的路径是否存在，如果不存在，则创建目录
			File zipTargetDir = new File(zipTargetDirPath);
			if (!zipTargetDir.exists()) {
				zipTargetDir.mkdirs();
			}
			
			logger.info("zipTargetDirPath----"+zipTargetDirPath+",zipTargetFileName="+zipTargetFileName);
			
			if (zipTargetFile.exists()) {
				// 重命名为备份文件 如果成功就删除备份文件  如果失败 重命名为zip文件
				zipTargetFile.renameTo(backupFile);
			}
			
			fos = new FileOutputStream(zipTargetFile);
			cos = new CheckedOutputStream(fos, new CRC32());
			zos = new ZipOutputStream(cos);

			// 调用递归压缩方法进行目录或文件压缩
			File projectSourceFile = new File(projectSourcePath);
			zip(projectSourcePath, projectSourceFile, zos);
			zos.flush();
		} catch (Exception e) {
			if (backupFile.exists()) {
				backupFile.renameTo(zipTargetFile);
			}
			e.printStackTrace();
			throw new RuntimeException("压缩"+projectSourcePath+"错误");
		} finally {
			
			try {
				if (zos != null) {
					zos.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

			if (cos != null) {
				try {
					cos.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			
			if (fos != null) {
				try {
					fos.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public static void main(String[] args) throws ZipException, IOException {
		
		/*String dir = "D:\\source";
		String zipPath = "d:\\target";
		String zipFileName = "test.zip";
		try {
			zip(dir, zipPath, zipFileName);
		} catch (Exception e) {
			e.printStackTrace();
		}*/
		
		/*String zipFilePath = "D:\\10target\\111\\azkaban_upload_project_111.zip";
		Map<String, Integer> listZipFile = listZipFile(zipFilePath,".job");
		System.err.println(listZipFile);*/
		
//		File file = new File("D:\\10\\target\\111\\azkaban_upload_project_111.zip");
		File file = new File("D:\\开发环境软件\\nginx-1.12.2.zip");
		System.err.println(file.exists());
		ZipFile zipFile = new ZipFile(file);
		Enumeration<? extends ZipEntry> zipEntrys = zipFile.entries();
		while(zipEntrys.hasMoreElements()){
			ZipEntry zipEntry = zipEntrys.nextElement();
			String fileName = zipEntry.getName();
			System.out.println("文件名"+fileName);
		}
		
	}
}